==> virtualfs-1.32.src.tar.gz: virtualfs-1.32/litefs/litefs-exec.cc <==
/*
	Execute a command on the back-end
*/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <pty.h>
#include <signal.h>
#include <sys/un.h>
#include "litefs.h"
#include "bufinput.h"

extern "C" char *stpcpy (char *, const char *);

static bool debug=false;

static void n_debug (const char *ctl, ...)
{
	if (debug){
		va_list list;
		va_start (list,ctl);
		vfprintf (stderr,ctl,list);
		va_end (list);
	}
}


static void usage()
{
	fprintf (stderr,"litefs-exec version 1.0\n"
		"\n"
		"litefs-exec [ --debug ] --socket unix_socket command [ args ...]\n");
	exit (-1);
}

static int fd = -1;
static int prog_handle = -1;

static void fct_signal (int nosig)
{
	n_debug ("Signal %d\n",nosig);
exit (0);
	LITE_REQUEST_DATA req;
	req.cmd = LITE_SIGNAL;
	req.client_handle = -1;
	req.handle = 0;
	req.mode = nosig;
	req.perm = 0;
	write (fd,&req,sizeof(LITE_REQUEST)+0);
}

static int litefsexec_loop ()
{
	int ret = -1;
	fd_set fdin;
	FD_ZERO (&fdin);
	FD_SET (0,&fdin);
	FD_SET (fd,&fdin);
	BUFINPUT_REPLY buffd (fd);
	while (1){
		fd_set in;
		memcpy (&in,&fdin,sizeof(in));
		n_debug ("select maxfd %d\n",fd+1);
		if (select (fd+1,&in,NULL,NULL,NULL)!=-1){
			if (FD_ISSET (fd,&in)){
				bool end = buffd.fill() <= 0;
				LITE_REPLY_DATA rep;
				while (buffd.read (rep)!=-1){
					n_debug ("read from front-end rep.status=%d,len=%d\n"
						,rep.status,rep.len);
					if (rep.status == LITE_PDATA_OUT){
						write (1,rep.buf,rep.len);
					}else if (rep.status == LITE_PDATA_ERR){
						write (2,rep.buf,rep.len);
					}else if (rep.status == LITE_STATUS_EXEC){
						n_debug ("Receive status_exec %d\n",rep.errcode);
						if (WIFEXITED(rep.errcode)){
							ret = WEXITSTATUS(rep.errcode);
						}
						end = true;
					}else if (rep.status == LITE_STATUS_OK){
						n_debug ("Receive status_ok\n");
						end = true;
					}else{
						fprintf (stderr,"litefs-exec: Protocol error (%d)\n",rep.status);
						end = true;
					}
				}
				if (end) break;
			}
			if (FD_ISSET (0,&in)){
				n_debug ("read stdin\n");
				LITE_REQUEST_DATA req;
				int len = read (0,req.buf,sizeof(req.buf));
				if (len <= 0){
					req.cmd = LITE_EXECIN;
					req.client_handle = -1;
					req.handle = 0;
					req.mode = 0;
					req.perm = 0;
					req.len = 0;
					write (fd,&req,sizeof(LITE_REQUEST)+0);
					FD_ZERO (&fdin);
					FD_SET (fd,&fdin);
				}else{
					req.cmd = LITE_EXECIN;
					req.client_handle = -1;
					req.handle = 0;
					req.mode = 0;
					req.perm = 0;
					req.len = len;
					write (fd,&req,sizeof(LITE_REQUEST)+req.len);
				}
			}
		}
	}
	return ret;
}

static struct termios origterm;
static void litefs_setterm()
{
	tcsetattr (0,TCSANOW,&origterm);
}


static int litefs_exec (const char *sockpath, char *arg[])
{
	int ret = -1;
	for (int i=0; i<NSIG; i++) signal 
(i,fct_signal);
	fd =  socket (AF_UNIX,SOCK_STREAM,0);
	if (fd == -1){
		perror("socket client");
	}else{
		n_debug ("socket open %d\n",fd);
		struct sockaddr_un un;
		un.sun_family = AF_UNIX;
		strcpy (un.sun_path,sockpath);
		int s = connect(fd,(struct sockaddr*)&un,sizeof(un));
		if (s == -1){
			perror("connect");
		}else{
			n_debug ("socket connected %d\n",s);
			LITE_REQUEST_DATA req;
			req.cmd = LITE_EXEC;
			req.mode = 0;
			req.perm = 0;
			req.handle = 0;
			req.client_handle = -1;
			LITE_EXECINFO_DATA *info = (LITE_EXECINFO_DATA*)req.buf;
			info->isatty0 = 0;
			if (isatty(1)){
				n_debug ("Running on a tty\n");
				// Ok, we are running on a tty, so we send the size
				// so the back-end can setup a proper pty
				struct winsize wins;
				if (ioctl(1,TIOCGWINSZ,&wins) != -1){
					info->win.height = wins.ws_row;
					info->win.width = wins.ws_col;
				}
				n_debug ("We have the size of the window\n");
				if (isatty(0)){
					n_debug ("Handle 0 is a tty\n");
					info->isatty0 = 1;
					struct termios term;
					tcgetattr (0,&term);
					tcgetattr (0,&origterm);
					atexit (litefs_setterm);
					cfmakeraw (&term);
					tcsetattr (0,TCSANOW,&term);
				}
				n_debug ("We have configured the ttys\n");
			}
			char *pt = info->data;
			info->nbarg = 0;
			for (int i=0; arg[i] != NULL; i++){
				pt = stpcpy (pt,arg[i]);
				pt++;
				info->nbarg++;
			}
			info->nbenv = 0;
			extern char **environ;
			for (int i=0; environ[i] != NULL; i++){
				const char *env = environ[i];
				if (strncmp(env,"TERM=",5)==0){
					info->nbenv++;
					pt = stpcpy (pt,env);
					pt++;
				}
			}
			req.len  = (int)(pt - (char*)info);
			n_debug ("Sending the request\n");
			ret = write (fd,&req,sizeof(LITE_REQUEST)+req.len);
			LITE_REPLY rep;
			int len = read (fd,&rep,sizeof(rep));
			if (len == sizeof(LITE_REPLY)){
				if (rep.status == LITE_STATUS_OK){
					prog_handle = rep.data;
					ret = litefsexec_loop ();
				}else{
				}
			}else{
			}
		}
		close (fd);
	}
	return ret;
}


int main (int argc, char *argv[])
{
	int ret = -1;
	if (argc == 1){
		usage();
	}else{
		const char *sock = NULL;
		int i;
		for (i=1; i<argc; i++){
			if (strcmp(argv[i],"--socket")==0){
				i++;
				sock = argv[i];
			}else if (strcmp(argv[i],"--debug")==0){
				debug = true;
			}else if (argv[i][0] == '-'){
				usage();
			}else{
				break;
			}
		}
		if (sock == NULL || i == argc){
			usage();
		}else{
			n_debug ("Executing command :%s:\n",argv[i]);
			ret = litefs_exec (sock, argv+i);
			n_debug ("Ret = %d\n",ret);
		}
	}
	return ret;
}

==> virtualfs-1.38.src.tar.gz: virtualfs-1.38/litefs/litefs-exec.cc <==
/*
	Execute a command on the back-end
*/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <pty.h>
#include <signal.h>
#include <sys/un.h>
#include "litefs.h"
#include "bufinput.h"

extern "C" char *stpcpy (char *, const char *);

static bool debug=false;

void n_debug (const char *ctl, ...)
{
	if (debug){
		static FILE *fout = NULL;
		if (fout == NULL) fout = fopen ("/tmp/litefs-exec.log","a");
		va_list list;
		va_start (list,ctl);
		vfprintf (fout != NULL ? fout : stderr,ctl,list);
		if (fout != NULL) fflush (fout);
		va_end (list);
	}
}


static void usage()
{
	fprintf (stderr,"litefs-exec version %s\n"
		"\n"
		"litefs-exec [ --debug ] --socket unix_socket command [ args ...]\n"
		,PACKAGE_REV);
	exit (-1);
}

static int fd = -1;
static int prog_handle = -1;

static void fct_signal (int nosig)
{
	n_debug ("Signal %d\n",nosig);
	//exit (0);
	LITE_REQUEST_DATA req;
	req.cmd = LITE_SIGNAL;
	req.client_handle = -1;
	req.handle = 0;
	req.mode = nosig;
	req.perm = 0;
	write (fd,&req,sizeof(LITE_REQUEST)+0);
}

static int litefsexec_loop ()
{
	int ret = -1;
	fd_set fdin;
	FD_ZERO (&fdin);
	FD_SET (0,&fdin);
	FD_SET (fd,&fdin);
	BUFINPUT_REPLY buffd (fd);
	while (1){
		fd_set in;
		memcpy (&in,&fdin,sizeof(in));
		n_debug ("select maxfd %d\n",fd+1);
		if (select (fd+1,&in,NULL,NULL,NULL)!=-1){
			if (FD_ISSET (fd,&in)){
				bool end = buffd.fill() <= 0;
				LITE_REPLY_DATA rep;
				while (buffd.read (rep)!=-1){
					n_debug ("read from front-end rep.status=%d,len=%d\n"
						,rep.status,rep.len);
					if (rep.status == LITE_PDATA_OUT){
						write (1,rep.buf,rep.len);
					}else if (rep.status == LITE_PDATA_ERR){
						write (2,rep.buf,rep.len);
					}else if (rep.status == LITE_STATUS_EXEC){
						n_debug ("Receive status_exec %d\n",rep.errcode);
						if (WIFEXITED(rep.errcode)){
							ret = WEXITSTATUS(rep.errcode);
						}
						end = true;
					}else if (rep.status == LITE_STATUS_OK){
						n_debug ("Receive status_ok\n");
						end = true;
					}else{
						fprintf (stderr,"litefs-exec: Protocol error (%d)\n",rep.status);
						end = true;
					}
				}
				if (end) break;
			}
			if (FD_ISSET (0,&in)){
				n_debug ("read stdin\n");
				LITE_REQUEST_DATA req;
				int len = read (0,req.buf,sizeof(req.buf));
				if (len <= 0){
					req.cmd = LITE_EXECIN;
					req.client_handle = -1;
					req.handle = 0;
					req.mode = 0;
					req.perm = 0;
					req.len = 0;
					write (fd,&req,sizeof(LITE_REQUEST)+0);
					FD_ZERO (&fdin);
					FD_SET (fd,&fdin);
				#if 0
					}else if (len == 1 && req.buf[0] == 3){
						// Control-C
						// fprintf (stderr,"req.buf %d\n",req.buf[0]);
						LITE_REQUEST_DATA sreq;
						sreq.cmd = LITE_SIGNAL;
						sreq.client_handle = -1;
						sreq.handle = 0;
						sreq.mode = SIGINT;
						sreq.perm = 0;
						write (fd,&sreq,sizeof(LITE_REQUEST)+0);
				#endif
				}else{
					req.cmd = LITE_EXECIN;
					req.client_handle = -1;
					req.handle = 0;
					req.mode = 0;
					req.perm = 0;
					req.len = len;
					write (fd,&req,sizeof(LITE_REQUEST)+req.len);
				}
			}
		}
	}
	return ret;
}

static struct termios origterm;
static void litefs_setterm()
{
	tcsetattr (0,TCSANOW,&origterm);
}


static int litefs_exec (const char *sockpath, char *arg[])
{
	int ret = -1;
	for (int i=0; i<NSIG; i++) signal (i,fct_signal);
	fd =  socket (AF_UNIX,SOCK_STREAM,0);
	if (fd == -1){
		perror("socket client");
	}else{
		n_debug ("socket open %d\n",fd);
		struct sockaddr_un un;
		un.sun_family = AF_UNIX;
		strcpy (un.sun_path,sockpath);
		int s = connect(fd,(struct sockaddr*)&un,sizeof(un));
		if (s == -1){
			perror("connect");
		}else{
			n_debug ("socket connected %d\n",s);
			LITE_REQUEST_DATA req;
			req.cmd = LITE_EXEC;
			req.mode = 0;
			req.perm = 0;
			req.handle = 0;
			req.client_handle = -1;
			LITE_EXECINFO_DATA *info = (LITE_EXECINFO_DATA*)req.buf;
			info->isatty0 = 0;
			if (isatty(1)){
				n_debug ("Running on a tty\n");
				// Ok, we are running on a tty, so we send the size
				// so the back-end can setup a proper pty
				struct winsize wins;
				if (ioctl(1,TIOCGWINSZ,&wins) != -1){
					info->win.height = wins.ws_row;
					info->win.width = wins.ws_col;
				}
				n_debug ("We have the size of the window\n");
				if (isatty(0)){
					n_debug ("Handle 0 is a tty\n");
					info->isatty0 = 1;
					struct termios term;
					tcgetattr (0,&term);
					tcgetattr (0,&origterm);
					atexit (litefs_setterm);
					cfmakeraw (&term);
					term.c_oflag |= OPOST;
					tcsetattr (0,TCSANOW,&term);
				}
				n_debug ("We have configured the ttys\n");
			}
			char *pt = info->data;
			info->nbarg = 0;
			for (int i=0; arg[i] != NULL; i++){
				pt = stpcpy (pt,arg[i]);
				pt++;
				info->nbarg++;
			}
			info->nbenv = 0;
			extern char **environ;
			for (int i=0; environ[i] != NULL; i++){
				const char *env = environ[i];
				if (strncmp(env,"TERM=",5)==0){
					info->nbenv++;
					pt = stpcpy (pt,env);
					pt++;
				}
			}
			req.len  = (int)(pt - (char*)info);
			n_debug ("Sending the request\n");
			ret = write (fd,&req,sizeof(LITE_REQUEST)+req.len);
			LITE_REPLY rep;
			int len = read (fd,&rep,sizeof(rep));
			if (len == sizeof(LITE_REPLY)){
				if (rep.status == LITE_STATUS_OK){
					prog_handle = rep.data;
					ret = litefsexec_loop ();
				}else{
					fprintf (stderr,"Can't execute %s\r\n",arg[0]);
				}
			}else{
				fprintf (stderr,"Can't execute, protocol error ?\n");
			}
		}
		close (fd);
	}
	return ret;
}


int main (int argc, char *argv[])
{
	int ret = -1;
	if (argc == 1){
		usage();
	}else{
		const char *sock = NULL;
		int i;
		for (i=1; i<argc; i++){
			if (strcmp(argv[i],"--socket")==0){
				i++;
				sock = argv[i];
			}else if (strcmp(argv[i],"--debug")==0){
				debug = true;
			}else if (argv[i][0] == '-'){
				usage();
			}else{
				break;
			}
		}
		if (sock == NULL || i == argc){
			usage();
		}else{
			n_debug ("Executing command :%s:\n",argv[i]);
			ret = litefs_exec (sock, argv+i);
			n_debug ("Ret = %d\n",ret);
		}
	}
	return ret;
}
